"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const microsoft_cognitiveservices_speech_sdk_1 = require("microsoft-cognitiveservices-speech-sdk");
const path_1 = tslib_1.__importDefault(require("path"));
const log_1 = require("../utils/log");
const defaultPaths_1 = require("../config/defaultPaths");
class TextToSpeechService {
    constructor(content) {
        this.voices = ['pt-BR-FranciscaNeural', 'pt-BR-AntonioNeural'];
        this.content = content;
        if (!process.env.AZURE_TTS_KEY) {
            log_1.error('Azure Key is not defined', 'TextToSpeechService');
            return;
        }
        if (!process.env.AZURE_TTS_REGION) {
            log_1.error('Azure Region is not defined', 'TextToSpeechService');
            return;
        }
        this.azureKey = process.env.AZURE_TTS_KEY;
        this.azureRegion = process.env.AZURE_TTS_REGION;
        this.content.renderData = [];
    }
    async execute({ synthesizeIntro, synthesizeEnd, }) {
        if (synthesizeIntro) {
            await this.synthesizeIntro();
        }
        await this.synthesizeNews();
        if (synthesizeEnd) {
            await this.synthesizeEnd();
        }
    }
    async synthesizeIntro() {
        var _a;
        if (!((_a = this.content.intro) === null || _a === void 0 ? void 0 : _a.text)) {
            log_1.log('Intro text is not defined, skipping...', 'TextToSeechService');
            return false;
        }
        if (typeof this.content.renderData !== 'object') {
            log_1.error('Render data is not defined', 'TextToSeechService');
            process.exit(1);
        }
        log_1.log(`Synthetizing Intro`, 'TextToSpeechService');
        const audioFilePath = await this.synthesize(this.content.intro.text, 'intro');
        this.content.renderData.push({
            text: this.content.intro.text,
            duration: 0,
            audioFilePath,
        });
        return true;
    }
    async synthesizeNews() {
        if (typeof this.content.renderData !== 'object') {
            log_1.error('Render data is not defined', 'TextToSeechService');
            process.exit(1);
        }
        let i;
        for (i = 0; i < this.content.news.length; i++) {
            log_1.log(`Synthetizing news ${i}`, 'TextToSpeechService');
            const audioFilePath = await this.synthesize(this.content.news[i].text, i.toString());
            this.content.renderData.push({
                text: this.content.news[i].text,
                duration: 0,
                audioFilePath,
            });
        }
        return i;
    }
    async synthesizeEnd() {
        var _a;
        if (!((_a = this.content.end) === null || _a === void 0 ? void 0 : _a.text)) {
            log_1.log('End text is not defined, skipping...', 'TextToSeechService');
            return;
        }
        if (typeof this.content.renderData !== 'object') {
            log_1.error('Render data is not defined', 'TextToSeechService');
            process.exit(1);
        }
        log_1.log('Synthetizing end', 'TextToSpeechService');
        const audioFilePath = await this.synthesize(this.content.end.text, 'end');
        this.content.renderData.push({
            text: this.content.end.text,
            duration: 0,
            audioFilePath,
        });
    }
    getVoice() {
        if (!this.voice) {
            const [voice] = this.voices;
            this.voice = voice;
        }
        else {
            const voiceIndex = this.voices.findIndex(voice => this.voice === voice);
            this.voice = this.voices[voiceIndex === 0 ? 1 : 0];
        }
        return this.voice;
    }
    async synthesize(text, sufix) {
        const tmpPath = await defaultPaths_1.getPath('tmp');
        return new Promise(resolve => {
            const outputFilePath = path_1.default.resolve(tmpPath, `output-${sufix}.mp3`);
            const speechConfig = microsoft_cognitiveservices_speech_sdk_1.SpeechConfig.fromSubscription(this.azureKey, this.azureRegion);
            speechConfig.speechSynthesisOutputFormat =
                microsoft_cognitiveservices_speech_sdk_1.SpeechSynthesisOutputFormat.Audio24Khz160KBitRateMonoMp3;
            const audioConfig = microsoft_cognitiveservices_speech_sdk_1.AudioConfig.fromAudioFileOutput(outputFilePath);
            const ssml = `
                <speak version="1.0" xml:lang="pt-BR">
                    <voice name="${this.getVoice()}">
                        <break time="250ms" /> ${text}
                    </voice>
                </speak>`;
            const synthesizer = new microsoft_cognitiveservices_speech_sdk_1.SpeechSynthesizer(speechConfig, audioConfig);
            synthesizer.speakSsmlAsync(ssml, _ => {
                synthesizer.close();
                resolve(outputFilePath);
            }, err => {
                synthesizer.close();
                log_1.error(`Synthesize of sentence ${sufix} failed. \n ${err}`, 'TextToSpeechService');
            });
        });
    }
}
exports.default = TextToSpeechService;
